@@ -1,16 +1,14 @@
 package App::wordlist;
 
 our $AUTHORITY = 'cpan:PERLANCAR'; # AUTHORITY
-our $DATE = '2020-05-01'; # DATE
+our $DATE = '2020-05-02'; # DATE
 our $DIST = 'App-wordlist'; # DIST
-our $VERSION = '0.267'; # VERSION
+our $VERSION = '0.268'; # VERSION
 
 use 5.010001;
 use strict;
 use warnings;
 
-use List::Util qw(shuffle);
-
 our %SPEC;
 
 our %arg_wordlists = (
@@ -45,6 +43,7 @@ sub _list_installed {
             list_modules  => 1,
             list_pod      => 0,
             recurse       => 1,
+            return_path   => 1,
         });
     my @res;
     for my $wl0 (sort keys %$mods) {
@@ -58,6 +57,7 @@ sub _list_installed {
         push @res, {
             name => $wl,
             lang => $lang,
+            path => $mods->{$wl0}{module_path},
         };
      }
     \@res;
@@ -91,11 +91,6 @@ $SPEC{wordlist} = {
             default => 0,
             cmdline_aliases => {n=>{}},
         },
-        random => {
-            summary => 'Pick random words (must set --num to positive number)',
-            schema  => 'bool*',
-            cmdline_aliases => {r=>{}},
-        },
         %arg_wordlists,
         or => {
             summary => 'Match any word in query instead of the default "all"',
@@ -243,15 +238,11 @@ sub wordlist {
     my $arg = $args{arg} // [];
     my $detail = $args{detail};
     my $num = $args{num} // 0;
-    my $random = $args{random};
     my $color = $args{color} // 'auto';
 
     my $use_color = ($color eq 'always' ? 1 : $color eq 'never' ? 0 : undef)
         // $ENV{COLOR} // (-t STDOUT);
 
-    return [412, "Must set --num to positive number when --random"]
-        if $random && !$num;
-
     if ($action eq 'grep') {
         # convert /.../ in arg to regex
         for (@$arg) {
@@ -276,11 +267,10 @@ sub wordlist {
                 push @$wordlists, $rec->{name};
             }
         }
-        $wordlists = [shuffle @$wordlists] if $random;
 
         my $n = 0;
 
-        my $code_add_word = sub {
+        my $code_format_word = sub {
             my ($wl, $word, $highlight_str, $ci) = @_;
             #use DD; dd \@_;
             if (defined $highlight_str) {
@@ -294,72 +284,69 @@ sub wordlist {
                     }
                 }
             }
-            if ($random) {
-                if (@res < $num) {
-                    splice @res, rand(@res+1), 0,
-                        $detail ? [$wl, $word] : $word;
-                } else {
-                    rand($n) < @res and splice @res, rand(@res), 1,
-                        $detail ? [$wl, $word] : $word;
-                }
-            } else {
-                push @res, $detail ? [$wl, $word] : $word;
-            }
+            $detail ? [$wl, $word] : $word;
         };
 
-        for my $wl (@$wordlists) {
-            my $mod;
-            {
-                my $modpm;
-                $mod = "WordList::$wl";
-                ($modpm = $mod . ".pm") =~ s!::!/!g;
-                last if eval { require $modpm; 1 };
-                die;
+        my $i_wordlist = 0;
+        my $wl_obj;
+        my $code_return_word = sub {
+          REDO:
+            return if $i_wordlist >= @$wordlists;
+            my $wl = $wordlists->[$i_wordlist];
+            unless ($wl_obj) {
+                my $mod = "WordList::$wl";
+                (my $modpm = "$mod.pm") =~ s!::!/!g;
+                eval { require $modpm; $wl_obj = $mod->new };
+                if ($@) {
+                    warn;
+                    $i_wordlist++;
+                    goto REDO;
+                }
+                $wl_obj->reset_iterator;
+            }
+            my $word = $wl_obj->next_word;
+            unless (defined $word) {
+                undef $wl_obj;
+                $i_wordlist++;
+                goto REDO;
             }
-            my $obj = $mod->new;
-            $obj->each_word(
-                sub {
-                    my $word = shift;
 
-                    return if !$random && $num > 0 && $n >= $num;
-                    return if defined($args{len}) &&
-                        _length_in_graphemes($word) != $args{len};
-                    return if defined($args{min_len}) &&
-                        _length_in_graphemes($word) < $args{min_len};
-                    return if defined($args{max_len}) &&
-                        _length_in_graphemes($word) > $args{max_len};
+            goto REDO if $num > 0 && $n >= $num;
+            goto REDO if defined($args{len}) &&
+                _length_in_graphemes($word) != $args{len};
+            goto REDO if defined($args{min_len}) &&
+                _length_in_graphemes($word) < $args{min_len};
+            goto REDO if defined($args{max_len}) &&
+                _length_in_graphemes($word) > $args{max_len};
 
-                    my $cmpword = $ci ? lc($word) : $word;
-                    my $match_arg;
-                    for (@$arg) {
-                        my $match =
-                            ref($_) eq 'Regexp' ? $cmpword =~ $_ :
-                                index($cmpword, $_) >= 0;
-                        if ($or) {
-                            # succeed early when --or
-                            if ($match) {
-                                $n++;
-                                $code_add_word->($wl, $word,
-                                                 $use_color ? $_ : undef, $ci);
-                                return;
-                            }
-                        } else {
-                            # fail early when and (the default)
-                            if (!$match) {
-                                return;
-                            }
-                        }
-                        $match_arg = $_;
-                    }
-                    if (!$or || !@$arg) {
+            my $cmpword = $ci ? lc($word) : $word;
+            my $match_arg;
+            for (@$arg) {
+                my $match =
+                    ref($_) eq 'Regexp' ? $cmpword =~ $_ :
+                    index($cmpword, $_) >= 0;
+                if ($or) {
+                    # succeed early when --or
+                    if ($match) {
                         $n++;
-                        $code_add_word->($wl, $word,
-                                         $use_color ? $match_arg : undef, $ci);
+                        return $code_format_word->(
+                            $wl, $word, $use_color ? $_ : undef, $ci);
+                    }
+                } else {
+                    # fail early when and (the default)
+                    if (!$match) {
+                        goto REDO;
                     }
                 }
-            );
-        }
-        [200, "OK", \@res];
+                $match_arg = $_;
+            }
+            if (!$or || !@$arg) {
+                $n++;
+                return $code_format_word->(
+                    $wl, $word, $use_color ? $match_arg : undef, $ci);
+            }
+        };
+        [200, "OK", $code_return_word, {stream=>1}];
 
     } elsif ($action eq 'list_installed') {
 
@@ -412,7 +399,7 @@ sub wordlist {
                     push @res, $mod unless grep {$mod eq $_} @res;
                 }
                 warn "Empty result from MetaCPAN\n" unless @res;
-                return [200, "OK", [$random ? shuffle(@res) : sort(@res)]];
+                return [200, "OK", \@res];
             }
         }
         return [412, "Can't find a way to list CPAN mirrors"];
@@ -439,7 +426,7 @@ App::wordlist - Grep words from WordList::*
 
 =head1 VERSION
 
-This document describes version 0.267 of App::wordlist (from Perl distribution App-wordlist), released on 2020-05-01.
+This document describes version 0.268 of App::wordlist (from Perl distribution App-wordlist), released on 2020-05-02.
 
 =head1 SYNOPSIS
 
@@ -550,10 +537,6 @@ Return (at most) this number of words (0 = unlimited).
 
 Match any word in query instead of the default "all".
 
-=item * B<random> => I<bool>
-
-Pick random words (must set --num to positive number).
-
 =item * B<wordlists> => I<array[str]>
 
 Select one or more wordlist modules.
@@ -572,6 +555,13 @@ that contains extra information.
 
 Return value:  (any)
 
+=head1 FAQ
+
+=head2 How to make wordlist return words in random order?
+
+The C<--random> (C<-r>) option was removed in v0.268. To return random words,
+you can pipe the output of C<wordlist> to C<shuf> or other similar utility.
+
 =head1 ENVIRONMENT
 
 =head2 DEBUG => bool
